Object subclass: #ScratchNotePlayer
	instanceVariableNames: 'midiPort channel volume midiKey snd '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scratch-Sound'!
!ScratchNotePlayer commentStamp: '<historical>' prior: 0!
I am a monophonic MIDI note player. I remember the midiPort, channel, volume, and currently playing note, if any. Playing a note will first turn off the note currently playing, if any.

!


!ScratchNotePlayer methodsFor: 'initialization' stamp: 'jm 5/6/2007 12:49'!
initialize

	midiPort _ nil.
	channel _ 1.
	volume _ 100.
	midiKey _ nil.
! !


!ScratchNotePlayer methodsFor: 'accessing' stamp: 'jm 8/7/2004 07:29'!
channel

	^ channel
! !

!ScratchNotePlayer methodsFor: 'accessing' stamp: 'jm 5/6/2007 13:16'!
channel: aNumber
	"Change my MIDI channel. This first turns off any notes in progress on the current channel."

	channel _ aNumber rounded within: 1 and: 16.
! !

!ScratchNotePlayer methodsFor: 'accessing' stamp: 'jm 6/2/2009 18:21'!
instrument: aNumber
	"Set the MIDI instrument number between 1 and 128. Instrument number 1 is piano. The instrument numbers are defined by the General MIDI specification; you can find the list of instruments on the web."

	| instr |
	instr _ aNumber rounded within: 1 and: 128.
	midiPort ifNotNil: [
		midiPort midiCmd: 16rC0 channel: (channel - 1) byte: (instr - 1)].
! !

!ScratchNotePlayer methodsFor: 'accessing' stamp: 'jm 8/9/2004 09:03'!
midiPort: aMIDIPort

	midiPort _ aMIDIPort.
! !

!ScratchNotePlayer methodsFor: 'accessing' stamp: 'jm 8/7/2004 07:29'!
volume

	^ volume
! !

!ScratchNotePlayer methodsFor: 'accessing' stamp: 'jm 12/14/2004 19:23'!
volume: aNumber
	"Set my volume in the range 0-100."

	volume _ aNumber.
! !


!ScratchNotePlayer methodsFor: 'playing' stamp: 'jm 6/2/2009 18:37'!
drumOff
	"Turn of the currently sounding drum, if any."

	midiKey ifNotNil: [
		midiPort ifNotNil: [midiPort midiCmd: 16r90 channel: 9 byte: midiKey byte: 0].
		midiKey _ nil].

	snd ifNotNil: [
		snd stopGracefully.
		snd _ nil].
! !

!ScratchNotePlayer methodsFor: 'playing' stamp: 'jm 6/3/2009 16:04'!
drumOn: aNumber
	"Play the given drum number at the current volume level. The drum number is defined by the General MIDI spec, which you can find on the web. The absolute value of the rounded drum number is used."

	| vol |
	midiKey _ aNumber rounded abs within: 0 and: 127.
	midiPort
		ifNil: [
			snd _ (SampledSound
					samples: SampledSound coffeeCupClink
					samplingRate: 11025) copy.
			snd
				setPitch: (AbstractSound pitchForMIDIKey: midiKey)
				dur: 1000
				loudness: ((volume / 500.0) within: 0.0 and: 1.0).
			snd play]
		ifNotNil: [
			vol _ (1.27 * volume) rounded within: 0 and: 127.
			midiPort midiCmd: 16r90 channel: 9 byte: midiKey byte: vol].
! !

!ScratchNotePlayer methodsFor: 'playing' stamp: 'jm 6/2/2009 18:38'!
noteOff
	"Turn of the currently sounding note, if any."

	midiKey ifNotNil: [
		midiPort ifNotNil: [midiPort midiCmd: 16r90 channel: (channel - 1) byte: midiKey byte: 0].
		midiKey _ nil].

	snd ifNotNil: [
		snd stopGracefully.
		snd _ nil].
! !

!ScratchNotePlayer methodsFor: 'playing' stamp: 'MJM 3/27/2011 05:18'!
noteOn: aNumber
	"Play the given note at the current volume level. The note number gives the piano key number where 60 is middle-C and there are 12 keys per octave.  The absolute value of the note number is used."

	| vol |
	midiKey _ aNumber rounded abs within: 0 and: 127.
	midiPort
		ifNil: [
			snd _ FMSound organ1.
			snd
				setPitch: (AbstractSound pitchForMIDIKey: midiKey)
				dur: 1000
				loudness: ((volume / 100.0) within: 0.0 and: 1.0).
			snd play]
		ifNotNil: [
			vol _ (1.27 * volume) rounded within: 0 and: 127.
			midiPort midiCmd: 144 channel: (channel - 1) byte: midiKey byte: vol].

! !
