DialogBoxMorph subclass: #EnchantingConfigurationDialog
	instanceVariableNames: 'palette blocksBin ports namesInUse '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Enchanting-Dialogs'!
!EnchantingConfigurationDialog commentStamp: '<historical>' prior: 0!
The EnchantingConfigurationDialog displays a dialog.  It allows you to configure parts of an EnchantingDeviceConfiguration object (or one of its subclasses).!


!EnchantingConfigurationDialog methodsFor: 'intialization' stamp: 'CWB 1/1/2011 15:35'!
buildBlockBinWith: compoundArray in: aColor
"The array (OrderedCollection) contains:
	an OrderedCollection of names for ports
	an OrderedCollection (of the same size) with configured morphs (or nil).
The compound array is ultimtely obtained by sending portsFor: on a descendant of EnchantingDeviceConfiguration."

	| portNames portBlocks total |

	portNames _ compoundArray first.
	portBlocks _ compoundArray last.
	
	(portNames size = portBlocks size) ifFalse: [self error: 'Both arrays in compoundArray must be the same size'].
	total _ portBlocks size.	
	ports _ OrderedCollection new.

	(1 to: total) do: [ :i |
		| name block position hatBlock |

		name _ portNames at: i.
		block _ portBlocks at: i.
		position _ self hatBlockPosition: i of: total.

		hatBlock _ ConfigurationHatMorph new 
			position: position;
			scriptOwner: blocksBin;
			title: name.
		blocksBin addMorphBack: hatBlock.
		ports add: hatBlock.	"Remember which port this hatblock represents"

		block ifNotNil: [
			block color: aColor.
			hatBlock nextBlock: block.
		]
	].	
! !

!EnchantingConfigurationDialog methodsFor: 'intialization' stamp: 'CWB 12/16/2010 12:57'!
buildPaletteWith: configBlocks
"Fills in the palette of blocks to use here"

	| x y |

	palette color: ScratchFrameMorph palettePaneColor.
	x _ 12.
	y _ 10.

	configBlocks do: [:blockOrSym |
		(blockOrSym = #-) | (blockOrSym = #~)
			ifTrue: [
				(blockOrSym = #-) ifTrue: [y _ y + 15].  "insert a full space"
				(blockOrSym = #~) ifTrue: [y _ y + 5]]  "insert a half space"
			ifFalse: [
				| block |
				block _ blockOrSym.
				block fixBlockLayout;
					position: x@y.
				palette addMorphBack: block.

				block placedInViewer.
				block submorphsDo: [:m | (m class canUnderstand: #placedInViewer) 
					ifTrue: [ m placedInViewer ]].
				block fixBlockLayout.

				y _ y + block height + 3. ]].


	
! !

!EnchantingConfigurationDialog methodsFor: 'intialization' stamp: 'CWB 1/1/2011 10:01'!
initialize

	| row |

	super initialize.
	self title: 'Configuration Editor' localized.

	palette _ ScratchBlockPaletteMorph new.	
	blocksBin _ ScratchScriptsMorph new.
	ports _ OrderedCollection new.
	peripheralNames _ Set new.

	"It appears that both palette and blocks bin need to be put inside a scrolling view,
	and the scrolling view needs to be put inside an object that will retain its size."
	{ palette . blocksBin } do: [:m |
		| scroller |
		scroller _ ScrollFrameMorph2 new
			growthFraction: 0.0;
			color: ScratchFrameMorph scriptsPaneColor;
			contents: m.
		ImageFrameMorph new
			initFrontFromForm: (ScratchFrameMorph skinAt: #dialogThumbnailFrame);
			color: Color red;
			addMorph: scroller ].

	row _ AlignmentMorph newRow
		color: Color transparent.

	row addMorphBack: palette owner owner;
		addMorphBack: (Morph new extent: 6@0);	"sizer"
		addMorphBack: blocksBin owner owner.

	mainColumn addMorph: row.

	self withButtonsForYes: false 
		no: false
		okay: true 
		cancel: true.
! !


!EnchantingConfigurationDialog methodsFor: 'geometry' stamp: 'CWB 11/9/2010 14:33'!
extent: aPoint

	| newExtent |

	newExtent _ aPoint max: self class minExtent.
	bounds extent = newExtent ifFalse: [
		self changed.
		bounds _ bounds topLeft extent: newExtent.
		self layoutChanged.
		self changed].

	self fixLayoutForExtent: newExtent.! !

!EnchantingConfigurationDialog methodsFor: 'geometry' stamp: 'CWB 12/11/2010 19:45'!
fixLayoutForExtent: aPoint

	| paletteWidth horizSpace vertSpace height palExt binExt |

	blocksBin ifNil: [^ self].

	paletteWidth _ 220.
	horizSpace _ 75.
	vertSpace _ titleBin bounds height + buttonRow bounds height + 22 + 16.
	height _ aPoint y - vertSpace.	

	"The palette and blocksBin are inside scrolling views, which are inside 
	a container that will retain its size.  We size the container and then the scroller."

	palExt _ paletteWidth @ height.
	palette owner owner extent: palExt.
	palette owner extent: palExt.

	binExt _ (aPoint x - paletteWidth - horizSpace) @ height.
	blocksBin owner owner extent: binExt.
	blocksBin owner extent: binExt.

	super fixLayoutForExtent: aPoint.
! !

!EnchantingConfigurationDialog methodsFor: 'geometry' stamp: 'CWB 12/20/2010 22:35'!
hatBlockPosition: index of: total
"Returns a position for a hat block.  The layout may depend on the total number of blocks"

"At the moment, the code does not take the total into account."

	| row col colWidth rowHeight baseX baseY numCols |

	((index <= 0) or: [index > total]) ifTrue: [self error: 'Index out of range.'].
	
	baseX _ 185.
	baseY _ 12.
	colWidth _ (218 - 12).
	rowHeight _ (130 - 12).
	numCols _ 2.	

	"We want to lay out blocks like so:
		1  2  3
         4  5  6
	That is to say, we go left to right and then up to down."

	col _ (index - 1) \\ numCols.
	row _ (index - 1) // numCols.
	
	^ (baseX + (col * colWidth)) @ (baseY + (row * rowHeight)).
! !


!EnchantingConfigurationDialog methodsFor: 'accessing' stamp: 'CWB 1/1/2011 21:35'!
blockAt: anIndex
	"Returns the configuration morph for the given port.  
	(Normally you want 'peripheralAt:' instead."

	| block |

	block _ (ports at: anIndex) nextBlock.
	^ block ifNotNil: [ block ] ifNil: [ nil ].
	! !

!EnchantingConfigurationDialog methodsFor: 'accessing' stamp: 'CWB 12/4/2010 21:20'!
peripheralAt: anIndex
"Returns the peripheral configured for the given port"

	| block |

	block _ (ports at: anIndex) nextBlock.
	^ block ifNotNil: [ block createPeripheral ] ifNil: [ nil ].
	! !


!EnchantingConfigurationDialog methodsFor: 'naming' stamp: 'CWB 9/22/2012 15:21'!
freeUpName: aName
"Removes the name from the list of names in use"

	aName isNil ifTrue: [ ^ self ].  "nil names aren't in the list"

	(namesInUse includes: aName) ifFalse: [ ^ self error: 'You may not free up a name that is not in use!!' ].

	namesInUse remove: aName.! !

!EnchantingConfigurationDialog methodsFor: 'naming' stamp: 'CWB 9/22/2012 15:20'!
namesInUse: theNamesInUse
"Provide a list of all the names of peripherals that are in use so they are not re-used"

	namesInUse _ (theNamesInUse reject: [:name | name isNil]) asSet.

	"Prevent some foolish names"
	namesInUse add: ChoiceArgMorphForPeripheral invalidContents.
	namesInUse add: '' asUTF8.

! !

!EnchantingConfigurationDialog methodsFor: 'naming' stamp: 'CWB 1/1/2011 11:26'!
registerName: aName
"Sent by a morph representing a peripheral to indicate that it is now using this name.  Call uniqueNameFor: first"

	(namesInUse includes: aName) ifTrue: [ ^ self error: 'You may not register a name that is already in use!!' ].

	namesInUse add: aName.! !

!EnchantingConfigurationDialog methodsFor: 'naming' stamp: 'CWB 1/1/2011 12:40'!
uniqueNameFor: aName
"Pass in a desired name and get a unique name back."

	| baseName newName suffix |

	(namesInUse includes: aName) ifFalse: [ ^ aName ].

	baseName _ aName withoutTrailingDigits.
	suffix _ 2.
	
	[ 	newName _ baseName, ' ', suffix asString. 
		namesInUse includes: newName 
	] whileTrue: [ suffix _ suffix + 1 ].

	^ newName.  	
	
	! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EnchantingConfigurationDialog class
	instanceVariableNames: ''!

!EnchantingConfigurationDialog class methodsFor: 'geometry' stamp: 'CWB 12/11/2010 19:42'!
minExtent

	^ 730 @ 420.! !
