ReporterBlockMorph subclass: #LabelFragmentBlockMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scratch-BYOB'!

!LabelFragmentBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/20/2010 11:32'!
addLabel
	"Add a label for my command and arguments. This is the method that really creates the whole block."
	"Details: We remove all the pieces of this block, and then reassemble them, being careful not to destroy any argument morphs that we might still want to use. This is helpful if someone adds/removes parameters from a hat block."

	| tokens args nextArgIndex m next |
	commandSpec ifNil: [^ self].

	"we make sure not to remove the next block."
	next _ self nextBlock.
	self removeAllMorphsIn: (self submorphs reject: [:t | t = next]).

	tokens _ self parseCommandSpecPreservingBlanks: commandSpec.
	args _ tokens select: [:s | CommandBlockMorph isArgSpec: s].

	"remove any excess arg morphs from the list."
	[argMorphs size > args size] whileTrue: [
		m _ argMorphs last.
		argMorphs remove: m.

		"if the argument we're throwing out is anything complicated, we'd like to keep it around in the world."
		(m isKindOf: BlockMorph) ifTrue: [m openInWorld]].

	"create new arg morphs."
	[argMorphs size < args size] whileTrue: [
		argMorphs addLast: (self argMorphFor: (args at: argMorphs size + 1))].

	nextArgIndex _ 1.

	tokens do: [:s |
		(CommandBlockMorph isArgSpec: s)
			ifTrue:	[
				m _ argMorphs at: nextArgIndex.
				nextArgIndex _ nextArgIndex + 1]
			ifFalse:	[
				m _ self labelMorphFor: s].
		self addMorphBack: m].

	self isReporter ifTrue: [

		"attempt to embed ReporterScripts more gracefully. Has some issues, 
		commented out for now, (jens)"
		
		"self adjustReporterLayoutForLambda."
		
		m _ submorphs last.
		((m isKindOf: ChoiceArgMorph) | (m isKindOf: SpriteArgMorph)) ifTrue: [  "add spacer"
			self addMorphBack: (Morph new color: Color transparent; extent: 2@5)]].
! !

!LabelFragmentBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 7/6/2010 23:01'!
drawOn: aCanvas 

	| right topY bottomY xInset c toggle l t cArgs clip |
	self computeHighlightColors.

	l _ OrderedCollection new.

	right _ self width.
	topY _ bottomY _ self height // 2.
	self height even ifTrue: [topY _ bottomY - 1].
	xInset _ 0.
	toggle _ 0.
	[topY >= 0] whileTrue: [
		topY = 0 ifTrue: [c _ highlightColor1].
		topY = 1 ifTrue: [c _ highlightColor2].
		topY > 1 ifTrue: [c _ color].
		self addHLineFrom: xInset to: right - xInset y: topY color: c to: l.
		(topY > 1) & (xInset > 0) ifTrue: [
			self addPoint: xInset@topY 
				color: (toggle < 2 ifTrue: [shadowColor] ifFalse: [color])
				to: l.
			self addPoint: (right - xInset - 1)@topY 
				color: (toggle < 2 ifTrue: [color] ifFalse: [shadowColor])
				to: l].

		c _ bottomY = (self height - 1) ifTrue: [shadowColor] ifFalse: [color].
		self addHLineFrom: xInset to: right - xInset y: bottomY color: c to: l.
		xInset > 0 ifTrue: [
			self addPoint: xInset@bottomY 
				color: (toggle > 1 ifTrue: [highlightColor1] ifFalse: [color])
				to: l.
			self addPoint: (right - xInset - 1)@bottomY 
				color: (toggle > 1 ifTrue: [color] ifFalse: [shadowColor])
				to: l].

		toggle < 2 
			ifTrue: [ 
				xInset _ xInset + 1.2.
				toggle _ toggle + 1 ]
			ifFalse: [
				xInset _ xInset - 1.
				toggle _ toggle + 1.
				toggle > 4 ifTrue: [
					toggle _ 0]].
		bottomY _ bottomY + 1.
		topY _ topY - 1].

	cArgs _ self nonControlFlowSubmorphs select: [:s | (s isKindOf: CArgSlotMorph) or: [s isKindOf: CReporterSlotMorph]].
	cArgs isEmpty 
		ifFalse: [
			cArgs do: [:ca |
				t _ OrderedCollection new.
				l do: [:pair |
					self addAreasOf: pair first outside: ca bounds color: pair second to: t].
			l _ t].
			t _ OrderedCollection new.
			clip _ (self right + 4 - (self height // 2) @ self top) corner: self bottomRight.
			l do: [:pair |
				self addAreasOf: pair first outside: clip color: pair second to: t].
			l _ t].
	
	l do: [:pair |
		aCanvas fillRectangle: pair first color: pair second ].
! !

!LabelFragmentBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 7/6/2010 23:04'!
fixBlockLayout
	self fixBlockLayoutNoWrap.
	(self submorphs select: [:m | (m isKindOf: CArgSlotMorph) or: [m isKindOf: CReporterSlotMorph]]) do: [:m |
		m width: 50]! !

!LabelFragmentBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/14/2011 01:58'!
parseCommandSpecPreservingBlanks: aCommandSpec
	"Answer an array of token strings containing my keywords and argument specs."

	| result len i j spec k |
	result _ OrderedCollection new.
	spec _ aCommandSpec.
	(spec isKindOf: UTF8) ifTrue: [spec _ String withAll: spec].
	len _ aCommandSpec size.

	i _ 1.
	[(i < len) and: [(spec at: i) isSeparator]] whileTrue: [i _ i + 1].
	[i <= len] whileTrue: [
		j _ spec indexOf: $% startingAt: i.
		j > 0
			ifTrue: [
				j > i ifTrue: [result addLast: (spec copyFrom: i to: j - 1)].
				j < len
					ifTrue: [k _ j + 1.
						((spec at: k) isLetter | (spec at: k) isDigit) 	"digits added for BYOB  -Jens"
							ifTrue: [[(k+1 <= len) and: [(spec at: k+1) isDigit]] whileTrue:[
								k _ k + 1].
								(k >= len)
									ifTrue: [result addLast: (spec copyFrom: j to: len).
										i _ len + 1]
									ifFalse: [result addLast: (spec copyFrom: j to: k).
										i _ k+1]]
							ifFalse: [result addLast: '%'.
										i _ j + 1]]
					ifFalse: [result addLast: '%'.
						i _ j + 2]]
			ifFalse: [
				result addLast: (spec copyFrom: i to: len).
				i _ len + 1]].

	result _ result select: [:s | s ~= ' '].
	^ result asArray collect: [:s | s copyReplaceAll: '_' with: ' ' asTokens: false]
! !
