package enchanting;

import java.awt.Rectangle;
import java.io.*;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.Sprite;
import javax.microedition.lcdui.Font;
import lejos.nxt.LCD;

public class Costume {
	float center_x;
	float center_y;
	Image mask;
	Image image;
	
	public Costume(float center_x, float center_y, Image mask, Image image) {
		this.center_x = center_x;
		this.center_y = center_y;
		this.mask = mask;
		this.image = image;
	}
	
	public Costume() {
		// Create a costume that can't really be used yet.
		this(0, 0, null, null);
	}
	
	// Loads a costume from a file. 
	public boolean loadCostume(String filename, int fileOffset) {
		// Load the costume from a file, using the provided starting point.
		try {
			FileInputStream fis = new FileInputStream(new File(filename));
			fis.skip(fileOffset);
			return loadCostume(fis);
		} catch (IOException e) {
			setToInvalidCostume();
			return false;
		}
	}
	
	// Loads a costume from a stream, and returns true if valid.
	public boolean loadCostume(InputStream s) {
	
		// Load a costume.
		// It will have one or two images,
		// and specify the center x and y coordinates 
		// before the image data.

		try {
			DataInputStream in = new DataInputStream(s);
			int imageCount = in.readShort();	
			if (imageCount != 1 && imageCount != 2) {
				setToInvalidCostume();
				return false;
			}

			this.center_x = in.readShort();
			this.center_y = in.readShort();
			
			if (imageCount == 1) {
				this.mask = null;
			} else { // imageCount == 2
				this.mask = Image.createImage(s);
			}
			
			this.image = Image.createImage(s);
			return true;
		} catch (IOException e) {
			setToInvalidCostume();
			return false;
		}
	}
	
	public void setToInvalidCostume() {
		this.mask = null;
		this.image = null;	
	}
	
	public boolean isInvalid() {
		return this.image == null;
	}
	
	public float getCenterX() { return center_x; }
	public float getCenterY() { return center_y; }
	public Image getMask()    { return mask; }
	public Image getImage()   { return image; }
	
	public int   getBubbleXOffset()		{ 
		if (image == null) return 0; 
		// return the right-most offset
		return image.getWidth() - (int)center_x; 
	}
	public int   getBubbleYOffset()		{ 
		if (image == null) return 0; 
		// return the top-most offset
		return -(int)center_y; 
	}
	
	// returns the bounding rectangle for the sprite,
	// given a starting position
	public Rectangle getBounds(int sprite_x, int sprite_y) {
		if (image == null) return new Rectangle(0, 0, 0, 0);
		int leftX  = sprite_x - (int)center_x;
		int topY   = sprite_y - (int)center_y;
		int width  = image.getWidth();
		int height = image.getHeight();		
	
		return new Rectangle(leftX, topY, width, height);
	}
	
	// Draws the costume on the current graphics context
	public void drawOn(Graphics g, int spriteX, int spriteY) {
		if (image == null) return;
			
		int leftX = spriteX - (int)center_x;
		int topY  = spriteY - (int)center_y;
		int width = image.getWidth();
		int height = image.getHeight();

		if (mask != null) {
			// Draw an image that has transparency
			g.drawRegionRop(mask, 0, 0, width, height, 
				leftX, topY, Graphics.LEFT | Graphics.TOP, LCD.ROP_AND);
			g.drawRegionRop(image, 0, 0, width, height, 
				leftX, topY, Graphics.LEFT | Graphics.TOP, LCD.ROP_OR);
		} else {
			// Draw an image that does not have transparency
			// (it is most likely the stage).
			g.drawImage(image, leftX, topY, Graphics.LEFT | Graphics.TOP);
		}
	}
	
	public void drawMaskOn(Graphics g, int spriteX, int spriteY, int rop) {
		if (mask == null) return;
		g.drawRegionRop(mask, 0, 0, 
			mask.getWidth(), mask.getHeight(), 
			spriteX - (int)center_x, spriteY - (int)center_y, 
			Graphics.LEFT | Graphics.TOP, 
			rop);
	}

	public void drawImageOn(Graphics g, int spriteX, int spriteY, int rop) {
		if (image == null) return;
		g.drawRegionRop(image, 0, 0, 
			image.getWidth(), image.getHeight(), 
			spriteX - (int)center_x, spriteY - (int)center_y, 
			Graphics.LEFT | Graphics.TOP, 
			rop);
	}

}